import json
import os.path
import sys
import inspect

cmd_subfolder = os.path.realpath(os.path.abspath(os.path.join(os.path.split(inspect.getfile( inspect.currentframe() ))[0], "ash-highlighter-lib")))
if cmd_subfolder not in sys.path:
    sys.path.insert(0, cmd_subfolder)

import click
from pygments import highlight
from pygments.formatter import Formatter
from pygments.lexers import guess_lexer, get_lexer_for_filename, get_lexer_by_name

# To get this to run, until we bundle everything together, you will need to get yourself "pip" and do:
# pip install pygments
# pip install click

PRETTY = False

class AshFormatter(Formatter):

    def __init__(self, **options):
        Formatter.__init__(self, **options)
        self.__charCount = 0
        self.__jsonIndent = 0
        self.__jsonSeparators = (',', ':')
        if PRETTY:
            self.__jsonIndent = 4
            self.__jsonSeparators = (',', ': ')


    def format(self, tokensource, outfile):
        lastval = ''
        lasttype = None
        first = True

        outfile.write('[')

        for ttype, value in tokensource:
            # We could consider simplifying further by using the parent token type, ttype.parent
            # eg: parent of Token.Literal.String.Double is Token.Literal.String

            if ttype == lasttype:
                lastval += value
            else:
                if lastval:

                    chunk = ''
                    if not first:
                        chunk += ','
                    if PRETTY:
                        chunk += '\n'

                    # AE doesn't like to count line breaks or tabs in selectors:
                    lastval = lastval.rstrip('\r\n\t').lstrip('\r\n\t')

                    if repr(lasttype) == 'Token.Text' and len(lastval.strip()) == 0:
                        offset = len(lastval.strip(' '))
                        self.__charCount -= offset

                    data = {'kind':  repr(lasttype),
                            'value': lastval,
                            'start': self.__charCount,
                            'end':   self.__charCount + len(lastval)}

                    chunk += json.dumps(data, indent=self.__jsonIndent, separators=self.__jsonSeparators)
                    if not PRETTY:
                        chunk = chunk.replace('\n', '')

                    # Omit Token.Text
                    if lastval and repr(lasttype) not in ('Token.Text', 'Token.Name'):
                        outfile.write(chunk)

                    first = False
                    self.__charCount += len(lastval)
                lastval = value
                lasttype = ttype

        outfile.write(PRETTY and '\n]\n' or ']')


@click.command()
@click.option('-i', '--infile', type=click.Path(exists=True), help='Input filepath')
@click.option('-o', '--outfile', help='Output filepath')
@click.option('-l', '--lang', default='', help='Language')
@click.option('--code-guess', default=False, is_flag=True, help='Guess from code.')
@click.option('--ext-guess', default=False, is_flag=True, help='Guess from filename.')
@click.option('--pretty', default=False, is_flag=True, help='Pretty JSON output. (Compact output otherwise.)')
def parse(infile, outfile, lang, code_guess, ext_guess, pretty):
    global PRETTY
    PRETTY = pretty

    f = open(infile, 'r')
    code = f.read()
    f.close()

    lexer = None
    if lang:
        lexer = get_lexer_by_name(lang)
    elif ext_guess:
        lexer = get_lexer_for_filename(os.path.basename(infile))
    elif code_guess:
        lexer = guess_lexer(code)

    if not lexer:
        raise Exception('NO LEXER FOUND!')

    print 'Colouring as', lexer.name

    prettyOutput = highlight(code, lexer, AshFormatter())
    if outfile:
        f = open(outfile, 'w')
        f.write(prettyOutput)
        f.close()
    else:
        print prettyOutput

    sys.exit(0)

if __name__ == '__main__':
    parse()
